// Quasi Multiport Server "I'm everywhere!".
// 2013-02-24 reaper7
// based on examples by jc@wippler.nl &
// last EtherCard github commit by mrbig:
// https://github.com/jcw/ethercard/commit/da563711872f14eb283dc0e6f6e9e95c1d9f0a5d
 
#include <EtherCard.h>
#include <SPI.h>
#include <EEPROM.h>

#define PAGEREFRESH	5	//reload page every ... [sec]

// ethernet mac address - must be unique on your network
static byte mymac[] = { 0x00,0x4f,0x49,0x00,0x00,0x04 };

byte Ethernet::buffer[500]; // tcp/ip send and receive buffer
BufferFiller bfill;

static int last_port=0;
static char buftext[30];

void homePage(uint16_t _pos, const char *_title, uint16_t _refresh) {
  long t = millis() / 1000;
  uint16_t h = t / 3600;
  byte m = (t / 60) % 60;
  byte s = t % 60;

	bfill = ether.tcpOffset();

	bfill.emit_p(
		"HTTP/1.0 200 OK\r\n"
		"Content-Type: text/html; charset=utf8\r\n"
		"Pragma: no-cache\r\n\r\n"
		"<!DOCTYPE HTML>\n"
		"<html>"
		"<head>"
		"<title>$F</title>"
		"<meta http-equiv='refresh' content='$D'/>"
		"</head>"
		"<body>"
		"<h1>$D$D:$D$D:$D$D</h1>"
		"</body>"
		"</html>",
		_title, _refresh, h/10, h%10, m/10, m%10, s/10, s%10
	);
	ether.httpServerReply(bfill.position());
}

void setup(){
	Serial.begin(115200);

  Serial.println("[Quasi Multiport Server]");
  
  //eg. ether.begin(sizeof Ethernet::buffer, mymac, PD_1, 3) //PD_1 - ENC CS Pin, SPImodule 3
  if (ether.begin(sizeof Ethernet::buffer, mymac) == 0)	//DEFAULT: PB_5 - ENC CS Pin, SPImodule 2
    Serial.println( "Failed to access Ethernet controller");
    
  if (!ether.dhcpSetup())
    Serial.println("DHCP failed");

  ether.printIp("IP:  ", ether.myip);
  ether.printIp("GW:  ", ether.gwip);  
  ether.printIp("DNS: ", ether.dnsip);  
}

void loop(){
	uint16_t pos;
	uint16_t plen = ether.packetReceive();

  // if valid tcp data is received...
  if (plen) {
    
    int _port = (((byte)Ethernet::buffer[TCP_DST_PORT_H_P] * 256) + ((byte)Ethernet::buffer[TCP_DST_PORT_L_P]));	//get request port number from eth buffer
    
		// prepare html title text & print remote ip + dst port to serial
    if ((_port) && (_port!=last_port)) {
      sprintf(buftext,"[EXT IP: %d.%d.%d.%d:%d]\0",Ethernet::buffer[IP_SRC_P],Ethernet::buffer[IP_SRC_P+1],Ethernet::buffer[IP_SRC_P+2],Ethernet::buffer[IP_SRC_P+3],_port);
      Serial.println(buftext);
      last_port=_port;
    }
    
    if (pos = ether.packetLoop(plen)) {							//default port 80
      homePage(pos, buftext, PAGEREFRESH);   
    }
    else if ((pos = ether.accept(_port, plen)) && (_port)) {	//all other ports
      homePage(pos, buftext, PAGEREFRESH); 
    }
  }
}
